@@ -29,9 +29,17 @@ has exclude_globs => (
     default => sub { $_[0]->config->exclude_file },
 );
 
+has include_globs => (
+    is      => 'rw',
+    lazy    => 1,
+    default => sub { $_[0]->config->include_file },
+);
+
 has status        => (is => 'rw', default => sub { 0 });
 has destfile_nice => (is => 'rw');
 
+our $found_comp;
+
 sub do {
     my ($self) = @_;
     $self->parse_copyrights or $self->make_orig_targz;
@@ -177,7 +185,7 @@ sub make_orig_targz {
               @{ $self->exclude_globs };
         };
         return $self->status(1) if ($@);
-        for my $filename (@files) {
+        for my $filename (sort @files) {
             my $last_match;
             for my $info (@exclude_info) {
                 if (
@@ -187,7 +195,27 @@ sub make_orig_targz {
 				(?:/.*)?$          # Possible trailing / for a directory
 			      @x
                 ) {
-                    $delete{$filename} = 1 if !$last_match;
+                    if (!$last_match) {
+                        # if the current entry is a directory, check if it
+                        # matches any exclude-ignored glob
+                        my $ignore_this_exclude = 0;
+                        for my $ignore_exclude (@{ $self->include_globs }) {
+                            my $ignore_exclude_regex
+                              = glob_to_regex($ignore_exclude);
+
+                            if ($filename =~ $ignore_exclude_regex) {
+                                $ignore_this_exclude = 1;
+                                last;
+                            }
+                            if (   $filename =~ m,/$,
+                                && $ignore_exclude =~ $info->{regex}) {
+                                $ignore_this_exclude = 1;
+                                last;
+                            }
+                        }
+                        next if $ignore_this_exclude;
+                        $delete{$filename} = 1;
+                    }
                     $last_match = $info;
                 }
             }
@@ -544,6 +572,12 @@ sub parse_copyrights {
                     grep { $_ }
                       split(/\s+/, $data->{ $self->config->excludestanza }));
             }
+            if ($data->{ $self->config->includestanza }) {
+                push(
+                    @{ $self->include_globs },
+                    grep { $_ }
+                      split(/\s+/, $data->{ $self->config->includestanza }));
+            }
         } else {
             if (open my $file, '<', $copyright_file) {
                 while (my $line = <$file>) {
@@ -572,8 +606,9 @@ sub fix_dest_file {
         $self->config->compression($comp
               || &Devscripts::MkOrigtargz::Config::default_compression);
     }
-    return sprintf "%s.%s", $destfiletar,
-      compression_get_property($self->config->compression, "file_ext");
+    $comp = compression_get_property($self->config->compression, "file_ext");
+    $found_comp ||= $self->config->compression;
+    return sprintf "%s.%s", $destfiletar, $comp;
 }
 
 1;
